package tictactoe;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import javax.swing.JComponent;
import javax.swing.JOptionPane;

/**
 *
 * @author Wave
 */
public class TicTacToe extends JComponent {

    public static final int FIELD_EMPTY = 0; //empty pole
    public static final int FIELD_X = 10; //pole s x
    public static final int FIELD_0 = 200; //pole s 0
    public static final int VEL_POLE = 3;
    int[][] field; // nas masiv hry pole
    boolean isXturn;

    public final void initGame() {
        for (int i = 0; i < VEL_POLE; ++i) {
            for (int j = 0; j < VEL_POLE; ++j) {
                field[i][j] = FIELD_EMPTY; // čistime masiv a davame tam 0
            }
        }
        isXturn = true;
        OccupiedTileException.setNumberOfClicks(0);
    }

    public TicTacToe() {
        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
        field = new int[VEL_POLE][VEL_POLE];
        initGame();
    }

    @Override
    public void processMouseEvent(MouseEvent mouseEvent) {
        try {
            super.processMouseEvent(mouseEvent);
            if (mouseEvent.getButton() == MouseEvent.BUTTON1 && mouseEvent.getID() == MouseEvent.MOUSE_CLICKED) {
                int x = mouseEvent.getX(); //koordinata x
                int y = mouseEvent.getY(); //koordinata y
                //převedeme koordinaty v indexu bunek v massivi field
                int i = (int) ((float) x / getWidth() * VEL_POLE);
                int j = (int) ((float) y / getHeight() * VEL_POLE);
                // kontrola jestli bunka prazdna a muže tam chodit
                if (field[i][j] == FIELD_EMPTY) {
                    //kontrola kod chodi
                    field[i][j] = isXturn ? FIELD_X : FIELD_0;
                    isXturn = !isXturn; //zmenime sled chodu (меняем порядок хода)
                    repaint();
                    int res = checkState();
                    if (res != 0) {
                        switch (res) {
                            case FIELD_0 * VEL_POLE:
                                //Vyhra 0
                                JOptionPane.showMessageDialog(this, "Výhra 0", "Výhra!", JOptionPane.INFORMATION_MESSAGE);
                                break;
                            case FIELD_X * VEL_POLE:
                                //Vyhra X
                                JOptionPane.showMessageDialog(this, "Výhra X", "Výhra!", JOptionPane.INFORMATION_MESSAGE);
                                break;
                            default:
                                JOptionPane.showMessageDialog(this, "Remíza", "Remíze", JOptionPane.INFORMATION_MESSAGE);
                                break;
                        }
                        //reload game
                        initGame();
                        //reload pole
                        repaint();
                    }
                } else {
                    /*JOptionPane.showMessageDialog(this, "Výberte jinou poličku", "Pozor!", JOptionPane.INFORMATION_MESSAGE);*/
                    throw new OccupiedTileException();
                }

            }
        } catch (OccupiedTileException e) {
            System.out.println(e.getMessage());
        }
    }

    @Override
    protected void paintComponent(Graphics graphics) {
        super.paintComponent(graphics);
        //čistime holst
        graphics.clearRect(0, 0, getWidth(), getHeight());
        //delame setku z linii
        drawGrid(graphics);
        //kreslime x 0
        drawXO(graphics);
    }

    void drawGrid(Graphics graphics) {  // šiřka pole
        int w = getWidth(); // šířka pole
        int h = getHeight(); // výška pole
        int dw = w / VEL_POLE; // delíme na 3 = jedno (jachejka)
        int dh = h / VEL_POLE; // delíme na 3 = jedno (jachejka)
        graphics.setColor(Color.BLUE); //barva linii
        for (int i = 1; i < VEL_POLE; i++) {
            graphics.drawLine(0, dh * i, w, dh * i); // horiyont linie
            graphics.drawLine(dw * i, 0, dw * i, h); // svisle linie (vertikal)

        }
    }

    void drawX(int i, int j, Graphics graphics) {
        graphics.setColor(Color.BLACK);
        int dw = getWidth() / VEL_POLE;
        int dh = getHeight() / VEL_POLE;
        int x = i * dw;
        int y = j * dh;
        //linie od vrchni levy uhel do prave nižní
        graphics.drawLine(x, y, x + dh, y + dh);
        //linie od leve nížní uhel do prave vrchni
        graphics.drawLine(x, y + dh, x + dw, y);
    }

    void draw0(int i, int j, Graphics graphics) {
        graphics.setColor(Color.BLACK);
        int dw = getWidth() / VEL_POLE;
        int dh = getHeight() / VEL_POLE;
        int x = i * dw;
        int y = j * dh;
        graphics.drawOval(x + 5 * dw / 100, y, dw * 9 / 10, dh);
    }

    void drawXO(Graphics graphics) {
        //vlkad cykla
        for (int i = 0; i < VEL_POLE; ++i) {
            for (int j = 0; j < VEL_POLE; ++j) {
                //jestli v bunce x - kreslime ho
                if (field[i][j] == FIELD_X) {
                    drawX(i, j, graphics);
                    //to same pro 0
                } else if (field[i][j] == FIELD_0) {
                    draw0(i, j, graphics);
                }
            }
        }
    }

    int checkState() {
        //diagonaly
        int diag = 0;
        int diag2 = 0;
        for (int i = 0; i < VEL_POLE; i++) {
            //suma po diagonal od leveho uhla
            diag += field[i][i];
            //suma po diagonal od praveho uhla
            diag2 += field[i][VEL_POLE - 1 - i];
        }
        //jestli v diagonale x nebo 0 vyhazeme z metody
        if (diag == FIELD_0 * VEL_POLE || diag == FIELD_X * VEL_POLE) {
            return diag;
        }
        //to same pro druhou diagonal
        if (diag2 == FIELD_0 * VEL_POLE || diag2 == FIELD_X * VEL_POLE) {
            return diag2;
        }
        int check_i, check_j;
        boolean hasEmpty = false;
        //bežíme po všem řadkam
        for (int i = 0; i < VEL_POLE; i++) {
            check_i = 0;
            check_j = 0;
            for (int j = 0; j < VEL_POLE; j++) {
                //sum znaku v řade
                if (field[i][j] == 0) {
                    hasEmpty = true;
                }
                check_i += field[i][j];
                check_j += field[j][i];
            }
            //jestli vyhra 0 nebo x, vyhazime
            if (check_i == FIELD_0 * VEL_POLE || check_i == FIELD_X * VEL_POLE) {
                return check_i;
            }
            if (check_j == FIELD_0 * VEL_POLE || check_j == FIELD_X * VEL_POLE) {
                return check_j;
            }
        }
        if (hasEmpty) {
            return 0;
        } else {
            return -1;
        }
    }
}
